@@ -16,51 +16,42 @@ if( defined( 'MEDIAWIKI' ) ) {
 # setting up a few globals.
 #
 
-global $wgProfiling, $wgProfileSampleRate, $wgIP, $wgUseSquid, $IP;
+// Check to see if we are at the file scope
+if ( !isset( $wgVersion ) ) {
+	die( "Error, Setup.php must be included from the file scope, after DefaultSettings.php\n" );
+}
 
 if( !isset( $wgProfiling ) )
 	$wgProfiling = false;
 
 if ( $wgProfiling and (0 == rand() % $wgProfileSampleRate ) ) {
-	require_once( 'Profiling.php' );
+        require_once( 'Profiling.php' );
 } else {
-	$wgFunctionStack = array();
-
-	if ( function_exists("setproctitle") ) {
-		function wfProfileIn( $fn = '' ) {
-			global $wgFunctionStack, $wgDBname;
-			$wgFunctionStack[] = $fn;
-			setproctitle($fn . " [$wgDBname]");
-		}
-		function wfProfileOut( $fn = '' ) {
-			global $wgFunctionStack, $wgDBname;
-			if (count($wgFunctionStack))
-				array_pop($wgFunctionStack);
-			if (count($wgFunctionStack))
-				setproctitle($wgFunctionStack[count($wgFunctionStack)-1] . " [$wgDBname]");
-		}
-	} else {
-		function wfProfileIn( $fn = '' ) {}
-		function wfProfileOut( $fn = '' ) {}
-	}
-	function wfGetProfilingOutput() {}
-	function wfProfileClose() {}
-}
-
-if ( $wgUseData ) {
-	$wgExtraNamespaces[20] = 'Data' ;
-	$wgExtraNamespaces[21] = 'Data_talk' ;
+        function wfProfileIn( $fn = '' ) {
+                global $hackwhere, $wgDBname;
+                $hackwhere[] = $fn;
+                if (function_exists("setproctitle"))
+                        setproctitle($fn . " [$wgDBname]");
+        }
+        function wfProfileOut( $fn = '' ) {
+                global $hackwhere, $wgDBname;
+                if (count($hackwhere))
+                        array_pop($hackwhere);
+                if (function_exists("setproctitle") && count($hackwhere))
+                        setproctitle($hackwhere[count($hackwhere)-1] . " [$wgDBname]");
+        }
+        function wfGetProfilingOutput( $s, $e ) {}
+        function wfProfileClose() {}
 }
 
 $fname = 'Setup.php';
 wfProfileIn( $fname );
-global $wgUseDynamicDates;
 wfProfileIn( $fname.'-includes' );
 
 require_once( 'GlobalFunctions.php' );
 require_once( 'Hooks.php' );
 require_once( 'Namespace.php' );
-require_once( 'RecentChange.php' ); 
+require_once( 'RecentChange.php' );
 require_once( 'User.php' );
 require_once( 'Skin.php' );
 require_once( 'OutputPage.php' );
@@ -72,12 +63,14 @@ require_once( 'Block.php' );
 require_once( 'MessageCache.php' );
 require_once( 'BlockCache.php' );
 require_once( 'Parser.php' );
-require_once( 'ParserXML.php' );
 require_once( 'ParserCache.php' );
 require_once( 'WebRequest.php' );
 require_once( 'LoadBalancer.php' );
 require_once( 'HistoryBlob.php' );
 require_once( 'ProxyTools.php' );
+require_once( 'ObjectCache.php' );
+require_once( 'WikiError.php' );
+require_once( 'SpecialPage.php' );
 
 if ( $wgUseDynamicDates ) {
 	require_once( 'DateFormatter.php' );
@@ -85,18 +78,6 @@ if ( $wgUseDynamicDates ) {
 
 wfProfileOut( $fname.'-includes' );
 wfProfileIn( $fname.'-misc1' );
-global $wgUser, $wgLang, $wgContLang, $wgOut, $wgTitle;
-global $wgLangClass, $wgContLangClass;
-global $wgArticle, $wgDeferredUpdateList, $wgLinkCache;
-global $wgMemc, $wgMagicWords, $wgMwRedir, $wgDebugLogFile;
-global $wgMessageCache, $wgUseMemCached, $wgUseDatabaseMessages;
-global $wgMsgCacheExpiry, $wgCommandLineMode;
-global $wgBlockCache, $wgParserCache, $wgParser, $wgMsgParserOptions;
-global $wgLoadBalancer, $wgDBservers, $wgDebugDumpSql;
-global $wgDBserver, $wgDBuser, $wgDBpassword, $wgDBname, $wgDBtype;
-global $wgUseOldExistenceCheck, $wgEnablePersistentLC, $wgMasterWaitTimeout;
-
-global $wgFullyInitialised;
 
 $wgIP = wfGetIP();
 $wgRequest = new WebRequest();
@@ -105,7 +86,7 @@ $wgRequest = new WebRequest();
 if ( $wgCommandLineMode ) {
 	# wfDebug( '"' . implode( '"  "', $argv ) . '"' );
 } elseif ( function_exists( 'getallheaders' ) ) {
-	wfDebug( "\nStart request\n" );
+	wfDebug( "\n\nStart request\n" );
 	wfDebug( $_SERVER['REQUEST_METHOD'] . ' ' . $_SERVER['REQUEST_URI'] . "\n" );
 	$headers = getallheaders();
 	foreach ($headers as $name => $value) {
@@ -116,86 +97,33 @@ if ( $wgCommandLineMode ) {
 	wfDebug( $_SERVER['REQUEST_METHOD'] . ' ' . $_SERVER['REQUEST_URI'] . "\n" );
 }
 
-# Disable linkscc except if the old existence check method is enabled
-if (!$wgUseOldExistenceCheck) {
-	$wgEnablePersistentLC = false;
+if ( $wgSkipSkin ) {
+	$wgSkipSkins[] = $wgSkipSkin;
 }
 
+$wgUseEnotif = $wgEnotifUserTalk || $wgEnotifWatchlist;
+
 wfProfileOut( $fname.'-misc1' );
 wfProfileIn( $fname.'-memcached' );
 
-# FakeMemCachedClient imitates the API of memcached-client v. 0.1.2.
-# It acts as a memcached server with no RAM, that is, all objects are
-# cleared the moment they are set. All set operations succeed and all
-# get operations return null.
+$wgMemc =& wfGetMainCache();
+$messageMemc =& wfGetMessageCacheStorage();
+$parserMemc =& wfGetParserCacheStorage();
 
-if( $wgUseMemCached ) {
-	# Set up Memcached
-	#
-	require_once( 'memcached-client.php' );
-	
-	/**
-	 *
-	 * @package MediaWiki
-	 */
-	class MemCachedClientforWiki extends memcached {
-		function _debugprint( $text ) {
-			wfDebug( "memcached: $text\n" );
-		}
-
-		function get( $key ) {
-			wfProfileIn( "memcached::get" );
-			$ret = memcached::get( $key );
-			wfProfileOut( "memcached::get" );
-			return $ret;
-		}
-	}
-
-	$wgMemc = new MemCachedClientforWiki( array('persistant' => true, 'compress_threshold' => 1500 ) );
-	$wgMemc->set_servers( $wgMemCachedServers );
-	$wgMemc->set_debug( $wgMemCachedDebug );
-
-	$messageMemc = &$wgMemc;
-} elseif ( $wgUseTurckShm ) {
-	# Turck shared memory
-	#
-	require_once( 'ObjectCache.php' );
-	$wgMemc = new TurckBagOStuff;
-	$messageMemc = &$wgMemc;
-} else {
-	/**
-	 * No shared memory
-	 * @package MediaWiki
-	 */
-	class FakeMemCachedClient {
-		function add ($key, $val, $exp = 0) { return true; }
-		function decr ($key, $amt=1) { return null; }
-		function delete ($key, $time = 0) { return false; }
-		function disconnect_all () { }
-		function enable_compress ($enable) { }
-		function forget_dead_hosts () { }
-		function get ($key) { return null; }
-		function get_multi ($keys) { return array_pad(array(), count($keys), null); }
-		function incr ($key, $amt=1) { return null; }
-		function replace ($key, $value, $exp=0) { return false; }
-		function run_command ($sock, $cmd) { return null; }
-		function set ($key, $value, $exp=0){ return true; }
-		function set_compress_threshold ($thresh){ }
-		function set_debug ($dbg) { }
-		function set_servers ($list) { }
-	}
-	$wgMemc = new FakeMemCachedClient();
-	
-	# Give the message cache a separate cache in the DB.
-	# This is a speedup over separately querying every message used
-	require_once( 'ObjectCache.php' );
-	$messageMemc = new MediaWikiBagOStuff('objectcache');
-}
+wfDebug( 'Main cache: ' . get_class( $wgMemc ) .
+       "\nMessage cache: " . get_class( $messageMemc ) .
+	   "\nParser cache: " . get_class( $parserMemc ) . "\n" );
 
 wfProfileOut( $fname.'-memcached' );
 wfProfileIn( $fname.'-SetupSession' );
 
-if( !$wgCommandLineMode && ( isset( $_COOKIE[ini_get('session.name')] ) || isset( $_COOKIE[$wgDBname.'Token'] ) ) ) {
+if ( $wgDBprefix ) {
+	session_name( $wgDBname . '_' . $wgDBprefix . '_session' );
+} else {
+	session_name( $wgDBname . '_session' );
+}
+
+if( !$wgCommandLineMode && ( isset( $_COOKIE[session_name()] ) || isset( $_COOKIE[$wgDBname.'Token'] ) ) ) {
 	User::SetupSession();
 	$wgSessionStarted = true;
 } else {
@@ -206,7 +134,7 @@ wfProfileOut( $fname.'-SetupSession' );
 wfProfileIn( $fname.'-database' );
 
 if ( !$wgDBservers ) {
-	$wgDBservers = array(array( 
+	$wgDBservers = array(array(
 		'host' => $wgDBserver,
 		'user' => $wgDBuser,
 		'password' => $wgDBpassword,
@@ -224,11 +152,43 @@ wfProfileIn( $fname.'-language1' );
 
 require_once( "$IP/languages/Language.php" );
 
+function setupLangObj($langclass) {
+	global $IP;
+
+	if( ! class_exists( $langclass ) ) {
+		# Default to English/UTF-8
+		$baseclass = 'LanguageUtf8';
+		require_once( "$IP/languages/$baseclass.php" );
+		$lc = strtolower(substr($langclass, 8));
+		$snip = "
+			class $langclass extends $baseclass {
+				function getVariants() {
+					return array(\"$lc\");
+				}
+
+			}";
+		eval($snip);
+	}
+
+	$lang = new $langclass();
+
+	return $lang;
+}
+
+# $wgLanguageCode may be changed later to fit with user preference.
+# The content language will remain fixed as per the configuration,
+# so let's keep it.
+$wgContLanguageCode = $wgLanguageCode;
+$wgContLangClass = 'Language' . str_replace( '-', '_', ucfirst( $wgContLanguageCode ) );
+
+$wgContLang = setupLangObj( $wgContLangClass );
+$wgContLang->initEncoding();
+
 wfProfileOut( $fname.'-language1' );
 wfProfileIn( $fname.'-User' );
 
 # Skin setup functions
-# Entries can be added to this variable during the inclusion 
+# Entries can be added to this variable during the inclusion
 # of the extension file. Skins can then perform any necessary initialisation.
 foreach ( $wgSkinExtensionFunctions as $func ) {
 	$func();
@@ -243,59 +203,25 @@ if( $wgCommandLineMode ) {
 	# Used for some maintenance scripts; user session cookies can screw things up
 	# when the database is in an in-between state.
 	$wgUser = new User();
+	# Prevent loading User settings from the DB.
+	$wgUser->setLoaded( true );
 } else {
-	$wgUser = User::loadFromSession();
+        $wgUser = null;
+	wfRunHooks('AutoAuthenticate',array(&$wgUser));
+	if ($wgUser === null) {
+		$wgUser = User::loadFromSession();
+	}
 }
 
 wfProfileOut( $fname.'-User' );
 wfProfileIn( $fname.'-language2' );
 
-function setupLangObj($langclass) {
-	global $wgUseLatin1, $IP;
-
-	if( ! class_exists( $langclass ) ) {
-		# Default to English/UTF-8
-		$baseclass = 'LanguageUtf8';
-		require_once( "$IP/languages/LanguageUtf8.php" );
-		$lc = strtolower(substr($langclass, 8));
-		$snip = "
-			class $langclass extends $baseclass {
-				function getVariants() {
-					return array(\"$lc\");
-				}
-
-			}";
-		eval($snip);
-	}
-
-	$lang = new $langclass();
-
-	if( ! $wgUseLatin1 ) 
-		return $lang;
-
-	require_once( $IP . '/languages/LanguageLatin1.php' );
-	$latin1 = new LanguageLatin1( $lang );
-	return $latin1;
-}
-
-# $wgLanguageCode may be changed later to fit with user preference.
-# The content language will remain fixed as per the configuration,
-# so let's keep it.
-$wgContLanguageCode = $wgLanguageCode;
-$wgContLangClass = 'Language' . str_replace( '-', '_', ucfirst( $wgContLanguageCode ) );
-
-$wgContLang = setupLangObj( $wgContLangClass );
-$wgContLang->initEncoding();
-
-// set default user option from content language
-if( !$wgUser->mDataLoaded ) {
-	$wgUser->loadDefaultFromLanguage();
-}
-
 // wgLanguageCode now specifically means the UI language
-$wgLanguageCode = $wgUser->getOption('language');
+$wgLanguageCode = $wgRequest->getText('uselang', '');
+if ($wgLanguageCode == '')
+	$wgLanguageCode = $wgUser->getOption('language');
 # Validate $wgLanguageCode, which will soon be sent to an eval()
-if( empty( $wgLanguageCode ) || !preg_match( '/^[a-z\-]*$/', $wgLanguageCode ) ) {
+if( empty( $wgLanguageCode ) || !preg_match( '/^[a-z]+(-[a-z]+)?$/', $wgLanguageCode ) ) {
 	$wgLanguageCode = $wgContLanguageCode;
 }
 
@@ -307,6 +233,7 @@ if( $wgLangClass == $wgContLangClass ) {
 	wfSuppressWarnings();
 	include_once("$IP/languages/$wgLangClass.php");
 	wfRestoreWarnings();
+
 	$wgLang = setupLangObj( $wgLangClass );
 }
 
@@ -314,7 +241,7 @@ wfProfileOut( $fname.'-language2' );
 wfProfileIn( $fname.'-MessageCache' );
 
 $wgMessageCache = new MessageCache;
-$wgMessageCache->initialise( $messageMemc, $wgUseDatabaseMessages, $wgMsgCacheExpiry, $wgDBname);
+$wgMessageCache->initialise( $parserMemc, $wgUseDatabaseMessages, $wgMsgCacheExpiry, $wgDBname);
 
 wfProfileOut( $fname.'-MessageCache' );
 
@@ -325,14 +252,14 @@ wfProfileOut( $fname.'-MessageCache' );
 # SEARCH INDEX UPDATES, AND MANY MANY THINGS.
 # DO NOT USE THIS MODE EXCEPT FOR TESTING RIGHT NOW.
 #
-# To disable it, the easiest thing could be to uncomment the 
+# To disable it, the easiest thing could be to uncomment the
 # following; they should effectively disable the UI switch functionality
 #
 # $wgLangClass = $wgContLangClass;
 # $wgLanguageCode = $wgContLanguageCode;
 # $wgLang = $wgContLang;
 #
-# TODO: Need to change reference to $wgLang to $wgContLang at proper 
+# TODO: Need to change reference to $wgLang to $wgContLang at proper
 #       places, including namespaces, dates in signatures, magic words,
 #       and links
 #
@@ -343,7 +270,6 @@ wfProfileOut( $fname.'-MessageCache' );
 wfProfileIn( $fname.'-OutputPage' );
 
 $wgOut = new OutputPage();
-wfDebug( "\n\n" );
 
 wfProfileOut( $fname.'-OutputPage' );
 wfProfileIn( $fname.'-BlockCache' );
@@ -361,8 +287,12 @@ $wgMagicWords = array();
 $wgMwRedir =& MagicWord::get( MAG_REDIRECT );
 $wgParserCache = new ParserCache( $messageMemc );
 
-if ( $wgUseXMLparser ) $wgParser = new ParserXML();
-else $wgParser = new Parser();
+if ( $wgUseXMLparser ) {
+	require_once( 'ParserXML.php' );
+	$wgParser = new ParserXML();
+} else {
+	$wgParser = new Parser();
+}
 $wgOut->setParserOptions( ParserOptions::newFromUser( $wgUser ) );
 $wgMsgParserOptions = ParserOptions::newFromUser($wgUser);
 wfSeedRandom();
@@ -375,13 +305,14 @@ wfProfileOut( $fname.'-misc2' );
 wfProfileIn( $fname.'-extensions' );
 
 # Extension setup functions for extensions other than skins
-# Entries should be added to this variable during the inclusion 
-# of the extension file. This allows the extension to perform 
+# Entries should be added to this variable during the inclusion
+# of the extension file. This allows the extension to perform
 # any necessary initialisation in the fully initialised environment
 foreach ( $wgExtensionFunctions as $func ) {
 	$func();
 }
 
+wfDebug( "\n" );
 $wgFullyInitialised = true;
 wfProfileOut( $fname.'-extensions' );
 wfProfileOut( $fname );
