@@ -28,29 +28,28 @@ class WantedPagesPage extends QueryPage {
 
 	function getSQL() {
 		$dbr =& wfGetDB( DB_SLAVE );
-		$brokenlinks = $dbr->tableName( 'brokenlinks' );
-
-		# We cheat and return the full-text from bl_to in the title.
-		# In the future, a pre-parsed name will be available.
-		$agrvalue=$dbr->aggregateValue('COUNT(DISTINCT bl_from)');
+		$pagelinks = $dbr->tableName( 'pagelinks' );
+		$page      = $dbr->tableName( 'page' );
 		return
-			"SELECT 'Wantedpages' as type,
-			        0 as namespace,
-			        bl_to as title,
-			        COUNT(DISTINCT bl_from) as value
-			FROM $brokenlinks
-			GROUP BY bl_to
-			HAVING $agrvalue > 1";
+			"SELECT 'Wantedpages' AS type,
+			        pl_namespace AS namespace,
+			        pl_title AS title,
+			        COUNT(*) AS value
+			 FROM $pagelinks
+			 LEFT JOIN $page
+			 ON pl_namespace=page_namespace AND pl_title=page_title
+			 WHERE page_namespace IS NULL
+			 GROUP BY pl_namespace,pl_title
+			 HAVING COUNT(*) > 1";
 	}
 
 	function formatResult( $skin, $result ) {
 		global $wgContLang;
 
-		$nt = Title::newFromDBkey( $result->title );
-		if( is_null( $nt ) ) {
-			return "<!-- Bad title '" . htmlspecialchars( $result->title ) . "' -->";
-		}
-		$plink = $skin->makeBrokenLink( $nt->getPrefixedText(), "" );
+		$nt = Title::makeTitle( $result->namespace, $result->title );
+		$text = $wgContLang->convert( $nt->getPrefixedText() );
+		$plink = $skin->makeBrokenLink( $nt->getPrefixedText(), $text );
+		
 		$nl = wfMsg( "nlinks", $result->value );
 		$nlink = $skin->makeKnownLink( $wgContLang->specialPage( "Whatlinkshere" ), $nl,
 		  "target=" . $nt->getPrefixedURL() );
