@@ -1,4 +1,8 @@
 <?php
+/**
+  * @package MediaWiki
+  * @subpackage Language
+  */
 
 # Stub for Urdu
 
@@ -7,7 +11,7 @@ require_once( "LanguageUtf8.php" );
 class LanguageUr extends LanguageUtf8 {
 
 	function getDefaultUserOptions() {
-		$opt = Language::getDefaultUserOptions();
+		$opt = parent::getDefaultUserOptions();
 		$opt["quickbar"] = 2; # Right-to-left
 		$opt["underline"] = 0; # Underline is hard to read in Arabic script
 		return $opt;
