@@ -8,6 +8,8 @@
  */ 
 
 /** */
+$wgUseMasterForMaintenance = true;
+$options = array( 'quick' );
 require_once( "commandLine.inc" );
 require_once( "updaters.inc" );
 $wgTitle = Title::newFromText( "MediaWiki database updater" );
@@ -15,12 +17,15 @@ $wgDatabase = Database::newFromParams( $wgDBserver, $wgDBadminuser, $wgDBadminpa
 
 print "Going to run database updates for $wgDBname\n";
 print "Depending on the size of your database this may take a while!\n";
-print "Abort with control-c in the next five seconds to aboard.. ";
 
-for ($i = 5; $i >= 0; --$i) {
-	echo $i;
-	sleep(1);
-	echo( ($i == 0) ? "\n" : chr(8) );
+if( !isset( $options['quick'] ) ) {
+	print "Abort with control-c in the next five seconds... ";
+	
+	for ($i = 5; $i >= 0; --$i) {
+		echo $i;
+		sleep(1);
+		echo( ($i == 0) ? "\n" : chr(8) );
+	}
 }
 
 do_all_updates();
